from simpletal import simpleTALES
from mod_python import util
import mx.DateTime

import logging

def handler(app, req):
	params = util.FieldStorage (req)
	if (not params.has_key ("requestAction")):
		# No action - use display
		action = "display"
	else:
		action = params ["requestAction"]
		
	displayer = Display (req, app, params)
	displayer.perform (action)
	
class Display:
	def __init__ (self, req, app, params):
		self.req = req
		self.app = app
		self.params = params
		self.db, self.dbToUnicode, self.unicodeToDb = app.getDBObjects()
		self.cursor = self.db.cursor()
		
	def perform (self, action):
		if (action == "display"):
			self.displayFeeds()
		elif (action == "displayEditFeed"):
			self.displayEditFeed()
		elif (action == "delete"):
			self.performDelete()
		elif (action == "update"):
			self.performUpdate()
		else:
			self.app.displayError (self.req, "Unsupported action %s" % action)
	
	def displayFeeds (self, notes=None):
		title = "Manage Available Feeds"
		context = simpleTALES.Context()
		feedContext = self._getUnusedFeeds_ ()
		if (len (feedContext) > 0):
			context.addGlobal ("unusedFeeds", feedContext)
		feedContext = self._getUsedFeeds_ ()
		if (len (feedContext) > 0):
			context.addGlobal ("usedFeeds", feedContext)
			
		context.addGlobal ("title", str (title))
		if (notes is not None):
			context.addGlobal ("notes", notes)
		template = self.app.getTemplate (self.app.getAppTemplateName("ADMIN_FEEDS"))
		self.app.displayTemplate (self.req, context, template)
		
	def displayEditFeed (self, notes=None):
		title = "Edit Feed"
		context = simpleTALES.Context()
		if (self.params.has_key ("feedID")):
			givenFeedID = self.params ["feedID"]
			feedContext = self._getFeedDetails_ (givenFeedID)
			context.addGlobal ("feed", feedContext)
			context.addGlobal ("title", title)
		else:
			self.displayFeeds ("WARNING: Internal error - no feedID provided.")
			return
		if (notes is not None):
			context.addGlobal ("notes", notes)
		template = self.app.getTemplate (self.app.getAppTemplateName("ADMIN_FEEDS_EDIT"))
		self.app.displayTemplate (self.req, context, template)
		
	def performDelete (self):
		if (self.params.has_key ("feedToDelete")):
			givenFeedIDs = self.params ["feedToDelete"]
			if (type (givenFeedIDs) == type ([])):
				feedList = givenFeedIDs
			else:
				feedList = [givenFeedIDs]
			for feedID in feedList:
				result = self._deleteFeed_ (feedID)
				if (result is not None):
					self.displayFeeds ("WARNING: %s" % result)
					return
		else:
			self.displayFeeds ("WARNING: No feeds selected for removal")
			return
		self.displayFeeds("Feeds deleted")
		
	def performUpdate (self):
		if (self.params.has_key ("feedID")):
			feedID = self.params ["feedID"]
		else:
			self.displayEditFeed ("WARNING: Internal error - no feedID found.")
			return
		if (self.params.has_key ("newURL")):
			newURL = self.params ["newURL"]
		else:
			self.displayEditFeed ("WARNING: URL not provided!")
			return
		if (self.params.has_key ("newFrequency")):
			try:
				newFrequency = int(self.params ["newFrequency"])
			except:
				self.displayEditFeed ("WARNING: Provided frequency is not a number!")
				return
		else:
			self.displayEditFeed ("WARNING: Check Frequency not provided!")
			return
		
		result = self._updateFeed_ (feedID, newURL, newFrequency)
		if (result is not None):
			self.displayEditFeed ("WARNING: %s" % result)
			return
		self.displayFeeds("Feed updated.")
		
	def _getUnusedFeeds_ (self):
		listUnusedFeeds = """SELECT af.feed_id, afd.title, af.feed_url, af.checkFrequency
							   FROM agg_feeds af
							 LEFT JOIN agg_user_feeds auf ON af.feed_id = auf.feed_id
							 LEFT JOIN agg_feed_details afd ON af.feed_id = afd.feed_id
							 WHERE auf.feed_id is null"""
		feedStats = """SELECT count(*)
							,min(articleDate)
							,max(articleDate)
						FROM agg_articles
						 WHERE feed_id = %s"""
		feedList = []
		self.cursor.execute (listUnusedFeeds)
		for row in self.cursor.fetchall():
			feedContext = {}
			self.cursor.execute (feedStats, row[0])
			stats = self.cursor.fetchone()
			feedContext ['feedID'] = str (row[0])
			feedContext ['title'] = self.dbToUnicode (row[1])
			feedContext ['url'] = row[2]
			feedContext ['totalArticles'] = str (stats[0])
			if (stats[1] is not None):
				feedContext ['firstArticle'] = stats[1].strftime ('%d %b %Y')
			if (stats[2] is not None):
				feedContext ['lastArticle'] = stats[2].strftime ('%d %b %Y')
			feedContext ['frequency'] = str (row[3])
			feedList.append (feedContext)
		return feedList
		
	def _getUsedFeeds_ (self):
		listUsedFeeds = """SELECT DISTINCT(af.feed_id), afd.title, af.feed_url, af.checkFrequency
						   FROM agg_feeds af
						 LEFT JOIN agg_user_feeds auf ON af.feed_id = auf.feed_id
						 LEFT JOIN agg_feed_details afd ON af.feed_id = afd.feed_id
						 WHERE auf.feed_id is not null"""
		feedStats = """SELECT count(*)
							,min(articleDate)
							,max(articleDate)
						FROM agg_articles
						WHERE feed_id = %s"""
		userNumbers = """SELECT count(*)
						 FROM agg_user_feeds
						WHERE feed_id = %s"""
		feedList = []
		self.cursor.execute (listUsedFeeds)
		for row in self.cursor.fetchall():
			feedContext = {}
			self.cursor.execute (feedStats, row[0])
			stats = self.cursor.fetchone()
			self.cursor.execute (userNumbers, row[0])
			userNum = self.cursor.fetchone()
			feedContext ['feedID'] = str (row[0])
			feedContext ['title'] = self.dbToUnicode (row[1])
			feedContext ['url'] = row[2]
			feedContext ['totalArticles'] = str (stats[0])
			if (stats[1] is not None):
				feedContext ['firstArticle'] = stats[1].strftime ('%d %b %Y')
			if (stats[2] is not None):
				feedContext ['lastArticle'] = stats[2].strftime ('%d %b %Y')
			feedContext ['numSubs'] = str (userNum[0])
			feedContext ['frequency'] = str (row[3])
			feedList.append (feedContext)
		return feedList
	
	def _deleteFeed_ (self, feedID):
		removeSubscriptions = """DELETE FROM agg_user_feeds WHERE feed_id = %s"""
		removeArticles = """DELETE FROM agg_articles WHERE feed_id = %s"""
		removeFeeds = """DELETE FROM agg_feeds WHERE feed_id = %s"""
		removeFeedDetails = """DELETE FROM agg_feed_details WHERE feed_id = %s"""
		
		try:
			logging.info ("Deleting feed %s from database" % str (feedID))
			result = self.cursor.execute (removeSubscriptions, feedID)
			logging.warn ("Removed %s subscriptions to this feed." % str (result))
			
			result = self.cursor.execute (removeArticles, feedID)
			logging.info ("Removed %s article belonging to this feed." % str (result))
			
			result = self.cursor.execute (removeFeeds, feedID)
			logging.info ("Removed the actual feed.")
			result = self.cursor.execute (removeFeedDetails, feedID)
			logging.info ("Removed the feed details.")
		except Exception, e:
			logging.error ("Error occured removing feed: " + str (e))
			return str (e)
			
	def _getFeedDetails_ (self, givenFeedID):
		feedDetails = """SELECT af.feed_url
							 ,af.checkFrequency
						FROM agg_feeds af
						WHERE af.feed_id = %s"""
		subscribedUsers = """SELECT userName
							 FROM agg_user_feeds auf
								 ,agg_users au
							WHERE auf.feed_id = %s
							  AND auf.user_id = au.user_id"""
		self.cursor.execute (feedDetails, givenFeedID)
		feedContext = {}
		feedAtts = self.cursor.fetchone()
		feedContext ['feedID'] = str(givenFeedID)
		feedContext ['url'] = feedAtts[0]
		feedContext ['checkFrequency'] = str(feedAtts[1])
		self.cursor.execute (subscribedUsers, givenFeedID)
		userList = []
		for row in self.cursor.fetchall():
			userList.append ({'userName': row[0]})
		if (len (userList) > 0):
			feedContext ['subscribers'] = userList
		return feedContext
		
	def _updateFeed_ (self, feedID, newURL, newFrequency):
		updateFeed = """UPDATE agg_feeds
						 SET feed_url = %s
							,checkFrequency = %s
						WHERE feed_id = %s"""
										 
		logging.info ("Updating feed ID: " + str (feedID))
		try:
			self.cursor.execute (updateFeed, (newURL, newFrequency, feedID))
		except Exception, e:
			logging.error ("Error updating feed: " + str (e))
			return str (e)
	
