from simpletal import simpleTALES
from mod_python import util
import mx.DateTime

import logging

def handler(app, req):
	params = util.FieldStorage (req)
	if (not params.has_key ("requestAction")):
		# No action - use display
		action = "display"
	else:
		action = params ["requestAction"]
		
	displayer = Display (req, app, params)
	displayer.perform (action)
	
class Display:
	def __init__ (self, req, app, params):
		self.req = req
		self.app = app
		self.params = params
		self.db, self.dbToUnicode, self.unicodeToDb = app.getDBObjects()
		self.cursor = self.db.cursor()
		
	def perform (self, action):
		if (action == "display"):
			self.displayUsers()
		elif (action == "displayChangePassword"):
			self.displayChangePassword()
		elif (action == "add"):
			self.performAdd()
		elif (action == "delete"):
			self.performDelete()
		elif (action == "update"):
			self.performUpdate()
		else:
			self.app.displayError (self.req, "Unsupported action %s" % action)
	
	def displayUsers(self, notes=None):
		title = "Manage Aggregator Users"
		context = simpleTALES.Context()
		usersContext = self._getUsers_ ()
		if (len (usersContext) > 0):
			context.addGlobal ("users", usersContext)
		context.addGlobal ("title", str (title))
		if (notes is not None):
			context.addGlobal ("notes", notes)
		template = self.app.getTemplate (self.app.getAppTemplateName("ADMIN_USERS"))
		self.app.displayTemplate (self.req, context, template)
		
	def displayChangePassword (self, notes=None):
		title = "Change users password"
		context = simpleTALES.Context()
		if (self.params.has_key ("userID")):
			userID = self.params ["userID"]
		else:
			self.displayUsers ("""WARNING: Internal error - no userID given""")
			return
			
		userContext = self._getUserDetails_ (userID)
		if (userContext is not None):
			context.addGlobal ("user", userContext)
		context.addGlobal ("title", str (title))
		if (notes is not None):
			context.addGlobal ("notes", notes)
		template = self.app.getTemplate (self.app.getAppTemplateName("ADMIN_USERS_PASSWD"))
		self.app.displayTemplate (self.req, context, template)
		
	def performAdd (self):
		if (self.params.has_key ("newUsername")):
			newUsername = self.params ["newUsername"]
		else:
			self.displayUsers ("""WARNING: No username given, so no user created""")
			return
		if (self.params.has_key ("newPassword")):
			newPassword = self.params ["newPassword"]
		else:
			self.displayUsers ("""WARNING: No password given, so no user created""")
			return
			
		result = self._addUser_ (newUsername, newPassword)
		if (result is not None):
			self.displayUsers ("WARNING: %s" % result)
			return
		else:
			self.displayUsers("""Created new user.""")

	def performUpdate (self):
		if (self.params.has_key ("userID")):
			userID = self.params ["userID"]
		else:
			self.displayUsers ("""WARNING: Internal error - no userID given""")
			return
		if (self.params.has_key ("newPassword")):
			newPassword = self.params ["newPassword"]
		else:
			self.displayUsers ("""WARNING: No password given, user not updated.""")
			return
			
		result = self._updateUser_ (userID, newPassword)
		if (result is not None):
			self.displayUsers ("WARNING: %s" % result)
			return
		else:
			self.displayUsers("""Updated password for user.""")
		
	def performDelete (self):
		if (self.params.has_key ("userToDelete")):
			givenUserIDs = self.params ["userToDelete"]
			if (type (givenUserIDs) == type ([])):
				userList = givenUserIDs
			else:
				userList = [givenUserIDs]
			for userID in userList:
				result = self._deleteUser_ (userID)
				if (result is not None):
					self.displayUsers ("WARNING: %s" % result)
					return
		else:
			self.displayUsers ("WARNING: No users selected for removal")
			return
		self.displayUsers("Deleted users")
		
	def _getUsers_(self):
		getUsers = """SELECT user_id, userName FROM agg_users ORDER BY userName"""
		getTotalSubscriptions = """SELECT COUNT(*) FROM agg_user_feeds WHERE user_id = %s"""
		self.cursor.execute (getUsers)
		list = []
		for row in self.cursor.fetchall():
			context = {}
			context ['userID'] = str(row[0])
			context ['userName'] = str(row[1])
			self.cursor.execute (getTotalSubscriptions, row[0])
			context ['subscribedFeeds'] = str (self.cursor.fetchone()[0])
			list.append (context)
		return list
	
	def _getUserDetails_ (self, userID):
		getUserName = """SELECT userName FROM agg_users WHERE user_id = %s"""
		context = {}
		self.cursor.execute (getUserName, userID)
		context ['userName'] = self.cursor.fetchone()[0]
		context ['userID'] = str (userID)
		return context
		
	def _addUser_ (self, newUsername, newPassword):
		createUser = """INSERT INTO agg_users (user_id
									,userName
									,password
									)
					VALUES (NULL
								 ,%s
								 ,PASSWORD(%s)
								 )"""
		try:
			self.cursor.execute (createUser, (newUsername, newPassword))
			return
		except Exception, e:
			return "Error creating new user: " + str (e)
			
	def _deleteUser_ (self, userID):
		deleteUsersFeeds = """DELETE FROM agg_user_feeds WHERE user_id = %s"""
		deleteUser = """DELETE FROM agg_users WHERE user_id = %s"""
		try:
			self.cursor.execute (deleteUsersFeeds, userID)
			self.cursor.execute (deleteUser, userID)
			return
		except Exception, e:
			return "Error deleting user: " + str (e)
	
	def _updateUser_ (self, userID, newPassword):
		updateUser = """UPDATE agg_users
											 SET password = PASSWORD(%s)
										 WHERE user_id = %s"""
		try:
			self.cursor.execute (updateUser, (newPassword, userID))
			return
		except Exception, e:
			return "Error updating password: " + str (e)
		
