from mod_python import apache
from mod_python import util
import os.path, os, stat, sys
from simpletal import simpleTAL, simpleTALES
import MySQLdb
import logging, logging.config
import AggregatorConfig
import codecs

# Attempt to read in the configuration for the system
AGG_CONFIG = AggregatorConfig.AggregatorConfig('/etc/aggregator/aggregator.ini')
	
# Application modules
import display, manageFeeds, manageTemplates, adminUsers, adminFeeds, index

DISPATCH_TABLE = {'display.py': display.handler
								 ,'manageFeeds.py': manageFeeds.handler
								 ,'manageTemplates.py': manageTemplates.handler
								 ,'adminUsers.py': adminUsers.handler
								 ,'index.py': index.handler
								 ,'adminFeeds.py': adminFeeds.handler
								 }
								 
ADMIN_PAGES = ['adminUsers.py', 'adminFeeds.py']

loggingConfig = AGG_CONFIG.getValue ('webclient.loggingConfig')
if (os.path.exists (loggingConfig)):
	logging.config.fileConfig (loggingConfig)
else:
	logging.basicConfig()
	
logging.info ("Aggregator mod python display.py loading.")
DB_CONN = MySQLdb.connect (db=AGG_CONFIG.getValue ('global.db')
												  ,user=AGG_CONFIG.getValue ('global.user')
												  ,passwd=AGG_CONFIG.getValue ('global.passwd')
												  )	

DEFAULT_TEMPLATES = {"ERROR_PAGE": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "error.html")
										,"INDEX_PAGE": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "index.html")
										,"MANAGE_FEEDS": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "manageFeeds.html")
										,"ADMIN_USERS": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "adminUsers.html")
										,"ADMIN_USERS_PASSWD": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "adminUsersPassword.html")
										,"ADMIN_FEEDS": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "adminFeeds.html")
										,"ADMIN_FEEDS_EDIT": os.path.join (AGG_CONFIG.getValue ('webclient.templatesDir'), "adminFeedsEdit.html")
										}

class App:
	def __init__ (self, db):
		self.templateCache = {}
		self.db = db

	def displayError (self, req, msg):
		context = simpleTALES.Context()
		context.addGlobal ("error", msg)
		errTemplate = self.getTemplate (self.getAppTemplateName("ERROR_PAGE"))
		self.displayTemplate (req, context, errTemplate)
		
	def getTemplate (self, name):
		if (self.templateCache.has_key (name)):
			template, oldctime = self.templateCache [name]
			ctime = os.stat (name)[stat.ST_CTIME]
			#logging.debug ("Old Ctime %s new Ctime %s" % (str (oldctime), str (ctime)))
			if (oldctime == ctime):
				logging.info ("Cache hit for template %s" % name)
				return template
				
		logging.info ("Cache miss for template %s" % name)	
		tempFile = open (name, 'r')
		if (name [-3:] == "xml"):
			template = simpleTAL.compileXMLTemplate (tempFile)
		else:
			template = simpleTAL.compileHTMLTemplate (tempFile)
		self.templateCache [name] = (template, os.stat (name)[stat.ST_CTIME])
		return template

	def displayTemplate (self, req, context, template):
		if (isinstance (template, simpleTAL.XMLTemplate)):
			req.content_type = "text/xml"
		else:
			req.content_type = "text/html"
		req.send_http_header()
		try:
			template.expand (context, req)
		except Exception, e:
			logging.error ("Error expanding template: " + str (e))
		
	def getDBObjects (self):
		return (self.db, AGG_CONFIG.dbToUnicode, AGG_CONFIG.unicodeToDb)
		
	def getAppTemplateName (self, name):
		return DEFAULT_TEMPLATES [name]
		
APP_OBJ = App (DB_CONN)

def handler(req):
	reload (display)
	reload (manageFeeds)
	try:
		handlerToUse = DISPATCH_TABLE[os.path.basename (req.filename)]		
	except Exception, e:
		APP_OBJ.displayError (req, "Error dispatching request: " + str (e))
	try:
		handlerToUse (APP_OBJ, req)
	except Exception, e:
		APP_OBJ.displayError (req, "Error processing request: " + str (e))
	return apache.OK

def adminAuthorised (user,pw):
	adminUser = AGG_CONFIG.getValue ('webclient.adminUsername')
	adminPass = AGG_CONFIG.getValue ('webclient.adminPassword')
	#logging.info ("Username/password: %s/%s" % (user,pw))
	if (user != adminUser) or (pw != adminPass):
		logging.warn ("Admin username/password do not match.")
		return 0
	logging.info ("Admin login accepted.")
	return 1
		
def authenhandler(req):
	pw = req.get_basic_auth_pw()
	user = req.connection.user
	logging.info ("File requested: " + req.filename)
	if (os.path.basename (req.filename) in ADMIN_PAGES):
		if (adminAuthorised (user,pw)):
			req.adminUser=1
			return apache.OK
		else:
			return apache.HTTP_UNAUTHORIZED
	cursor = APP_OBJ.getDBObjects()[0].cursor()
	loginStmt = """SELECT user_id FROM agg_users WHERE userName=%s AND password=PASSWORD(%s)"""
	cursor.execute (loginStmt, (user, pw))
	result = cursor.fetchone()
	if (result is None):
		# Check to see if it's the admin.
		if (adminAuthorised (user,pw)):
			req.adminUser=1
			return apache.OK
		return apache.HTTP_UNAUTHORIZED
	req.agg_user_id = result[0]
	req.agg_user_name = user
	return apache.OK
