from simpletal import simpleTALES
from mod_python import util
import logging

def handler(app, req):
	params = util.FieldStorage (req)
	if (not params.has_key ("template")):
		templateID = 1
	else:
		templateID = params ["template"]
		
	if (not params.has_key ("page")):
		page = 0
	else:
		page = int (params ["page"])
	user = req.agg_user_id
	displayer = Display (req, app, user,templateID, page)
	displayer.displayArticles()
	
class Display:
	def __init__ (self, req, app, userID, templateID, page):
		self.req = req
		self.app = app
		self.userID = userID
		self.templateID = templateID
		self.page = page
		self.db, self.dbToUnicode, self.unicodeToDb = app.getDBObjects()
		self.cursor = self.db.cursor()
		
	def displayArticles (self):
		if (not self._getTemplateDetails_()):
			self.app.displayError (self.req, "Error retrieveing template information.")
			self.cursor.close()
			return
		try:
			contextList = self._getArticles_ ()
		except Exception, e:
			self.app.displayError (self.req, "Error getting list of latest articles: %s" % str (e))
			self.cursor.close()
			return
		# Get the template to display the feeds through
		try:
			template = self.app.getTemplate (self.templateFile)
		except Exception, e:
			self.app.displayError (self.req, "Error loading template from file %s.  Exception: %s" % (self.templateFile, str (e)))
			self.cursor.close()
			return
		
		context = simpleTALES.Context()
		if (len (contextList) > 0):
			context.addGlobal ("articles", contextList)
		if (self.page > 0):
			context.addGlobal ("previousPage", str(self.page - 1))
		if (len (contextList) == self.numEntries):
			context.addGlobal ("nextPage", str(self.page + 1))
		context.addGlobal ("currentDisplayPage", str (self.page + 1))
		context.addGlobal ("userName", self.req.agg_user_name)
		context.addGlobal ("templateID", str (self.templateID))
		context.addGlobal ("cgiLocation", self.req.uri)
		self.app.displayTemplate (self.req, context, template)
		self.cursor.close()
		
	def _getTemplateDetails_ (self):
		getStmt = """SELECT user_id, templateFile, numEntries FROM agg_users_templates WHERE template_id = %s"""
		self.cursor.execute (getStmt, str (self.templateID))
		result = self.cursor.fetchone()
		if (result is None):
			logging.warn ("No template found for ID: " + str (self.templateID))
			return 0
		if (result[0] is not None and (result[0] != self.userID)):
			logging.warn ("User %s requested template %s, but this belongs to %s: " % (str (self.userID), str (self.templateID), str (result[0])))	
			return 0
			
		self.templateFile = result[1]
		self.numEntries = result[2]
		return 1
		
	def _getArticles_ (self):
		getStmt = """SELECT a.article_id, a.articleDate, a.discoveryDate,a.title,a.description,a.htmlDescription,a.link
											 ,a.feed_id, f.title, f.imageURL, f.imageLink, f.imageTitle, f.channelLink, f.channelDesc, f.htmlChannelDesc, f.language, f.goodXML
											 ,u.userLabel
 									FROM agg_articles a, agg_feed_details f, agg_user_feeds u
  								WHERE a.feed_id = f.feed_id
  								  AND u.feed_id = f.feed_id
  								  AND u.user_id = %s
  								  AND a.firstFeedFlag = 'N'
 								 ORDER BY a.discoveryDate desc
 								         ,a.article_id asc
 								 LIMIT %s,%s"""
 		# NOTE: We order by article_id asc so that un-dated articles are in RSS order
		startPage = self.page * self.numEntries
		endPage = self.numEntries
		self.cursor.execute (getStmt, (str (self.userID), startPage, endPage))
		contextList = []
		lastDate = None
		for row in self.cursor.fetchmany (self.numEntries):
			context = {}
			articleDets = ArticleDetails (row[0], row[1], row[2])
			if (row[3] is not None):
				articleDets.setTitle (self.dbToUnicode (row[3]))
			if (row[4] is not None):
				#logging.warn ("Description is type: " + repr (row[4]))
				articleDets.setDescription (self.dbToUnicode (row[4]), self.dbToUnicode (row[5]))
			if (row[6] is not None):
				articleDets.setLink (row[6])
			
			feedDets = FeedDetails (row[7])
			if (row[8] is not None):
				feedDets.setTitle (self.dbToUnicode (row[8]))
			if (row[9] is not None):
				feedDets.setImageURL (row[9])
			if (row[10] is not None):
				feedDets.setImageLink (row[10])
			if (row[11] is not None):
				feedDets.setImageTitle (self.dbToUnicode (row[11]))
			if (row[12] is not None):
				feedDets.setLink (row[12])
			if (row[13] is not None):
				feedDets.setDescription (self.dbToUnicode (row[13]), self.dbToUnicode (row[14]))
			if (row[15] is not None):
				feedDets.setLanguage (row[15])
			if (row[16] is not None):
				feedDets.setGoodXML(row[16])
			if (row[17] is not None):
				feedDets.setLabel(row[17])
			#logging.debug ("Appending context: " + str (context))
			if (articleDets.discoveryDate.day % 2):
				dateEven = 0
			else:
				dateEven = 1
			if ((lastDate is None) or (articleDets.discoveryDate.day != lastDate.day)):
				newDate = 1
				lastDate = articleDets.discoveryDate
			else:
				newDate = 0
			
			context ["feed"] = feedDets.getContext()
			context ["item"] = articleDets.getContext()
			context ["displayInfo"] = {'dateEven': dateEven, 'newDate': newDate}
			contextList.append (context)
		return contextList

class ArticleDetails:
	def __init__ (self, id, articleDate, discoveryDate):
		self.context = {'articleID': id, 'dateLong': discoveryDate.strftime('%A, %d %B %Y'), 'time24Hour': discoveryDate.strftime('%H:%M'),'discoveryDate': discoveryDate.strftime()}
		self.description = None
		self.safeDescription = None
		self.articleDate = articleDate
		self.discoveryDate = discoveryDate
		
	def setTitle (self, title):
		self.context ['shortTitle'] = title[0:20]
		self.context ['mediumTitle'] = title[0:80]
		self.context ['fullTitle'] = title
		
	def setDescription (self, description, htmlDescription):
		self.context ['shortDescription'] = description[0:80]
		self.context ['mediumDescription'] = description[0:249]
		self.context ['fullDescription'] = description
		self.context ['htmlDescription'] = htmlDescription
		
	def setLink (self, link):
		self.context ['link'] = link
	
	def getContext (self):
		return self.context


class FeedDetails:
	def __init__ (self, id):
		self.context = {'feedID': id}
		self.description = None
		self.safeDescription = None
		
	def setTitle (self, title):
		self.context ['shortTitle'] = title[0:20]
		self.context ['mediumTitle'] = title[0:80]
		self.context ['fullTitle'] = title
		
	def setDescription (self, description, htmlDescription):
		self.context ['shortDescription'] = description[0:80]
		self.context ['mediumDescription'] = description[0:249]
		self.context ['fullDescription'] = description
		self.context ['htmlDescription'] = htmlDescription
		
	def setLabel (self, label):
		self.context ['label'] = label
		
	def setLink (self, link):
		self.context ['link'] = link
		
	def setImageURL (self, link):
		self.context ['imageURL'] = link
		
	def setImageLink (self, link):
		self.context ['imageLink'] = link
		
	def setImageTitle (self, title):
		self.context ['imageTitle'] = title
		
	def setLanguage (self, lang):
		self.context ['language'] = lang
		
	def setGoodXML (self, flag):
		if (flag == 'Y'):
			self.context ['goodXML'] = 'Y'
			
	def getContext (self):
		return self.context


