from simpletal import simpleTALES
from mod_python import util
import mx.DateTime

import logging

def handler(app, req):
	params = util.FieldStorage (req)
	user = req.agg_user_id
	if (not params.has_key ("action")):
		# No action - use display
		action = "display"
	else:
		action = params ["action"]
		
	displayer = Display (req, app, user, params)
	displayer.perform (action)
	
class Display:
	def __init__ (self, req, app, userID, params):
		self.req = req
		self.app = app
		self.userID = userID
		self.params = params
		self.db, self.dbToUnicode, self.unicodeToDb = app.getDBObjects()
		self.cursor = self.db.cursor()
		
	def perform (self, action):
		if (action == "display"):
			self.displayFeeds()
		elif (action == "add"):
			self.performAdd()
		elif (action == "delete"):
			self.performDelete()
		else:
			self.app.displayError (self.req, "Unsupported action %s" % action)
	
	def displayFeeds(self, notes=None):
		title = "Manage Subscribed Feeds"
		context = simpleTALES.Context()
		feedContext = self._getFeeds_ ()
		if (len (feedContext) > 0):
			context.addGlobal ("feeds", feedContext)
		context.addGlobal ("userID", str (self.userID))
		context.addGlobal ("userName", self.req.agg_user_name)
		context.addGlobal ("title", str (title))
		if (notes is not None):
			context.addGlobal ("notes", notes)
		template = self.app.getTemplate (self.app.getAppTemplateName("MANAGE_FEEDS"))
		self.app.displayTemplate (self.req, context, template)
		
	def performAdd (self):
		if (self.params.has_key ("newFeedURL")):
			urlToAdd = self.params ["newFeedURL"]
			if (self.params.has_key ("newFeedLabel")):
				label = self.params ["newFeedLabel"]
			else:
				label = None
			result = self._addFeed_ (urlToAdd, label)
			if (result is not None):
				self.displayFeeds ("WARNING: %s" % result)
				return
			else:
				self.displayFeeds("""Added feed %s to list of subscribed feeds.  Please allow a few hours
														for the feed to be read and parsed.""" % urlToAdd)
		else:
			self.displayFeeds ("""WARNING: URL Not added - no URL entered.""")
		
	def performDelete (self):
		if (self.params.has_key ("feedToDelete")):
			givenFeedIDs = self.params ["feedToDelete"]
			logging.debug ("Param: " + str (givenFeedIDs))
			if (type (givenFeedIDs) == type ([])):
				feedList = givenFeedIDs
			else:
				feedList = [givenFeedIDs]
			for feedID in feedList:
				result = self._deleteFeed_ (feedID)
				if (result is not None):
					self.displayFeeds ("WARNING: %s" % result)
					return
		else:
			self.displayFeeds ("WARNING: No feeds selected.")
			return
		self.displayFeeds("Deleted feeds")
		
	def _getFeeds_(self):
		getStmt = """SELECT auf.feed_id, auf.userLabel, auf.subscriptionDate, af.feed_url, af.checkFrequency, af.nextCheck FROM agg_user_feeds auf, agg_feeds af WHERE user_id = %s AND auf.feed_id = af.feed_id ORDER BY af.feed_id"""
		self.cursor.execute (getStmt, str (self.userID))
		list = []
		for row in self.cursor.fetchall():
			context = {}
			if (row[0] is not None):
				context ['feedID'] = str(row[0])
			if (row[1] is not None):
				context ['userLabel'] = self.dbToUnicode (row[1])
			if (row[2] is not None):
				context ['subscriptionDate'] = row[2].strftime()
			if (row[3] is not None):
				context ['feedURL'] = row[3]
			if (row[4] is not None):
				context ['checkFrequency'] = row[4]
			if (row[5] is not None):
				context ['nextCheck'] = row[5].strftime()
			list.append (context)
		return list
	
	def _addFeed_ (self, urlToAdd, label):
		findExistingFeed = """SELECT feed_id FROM agg_feeds WHERE feed_url=%s"""
		self.cursor.execute (findExistingFeed, (urlToAdd,))
		result = self.cursor.fetchone()
		if (result is None):
			logging.info ("No existing feed for url %s found - adding a new one!" % urlToAdd)
			addFeedStmt = """INSERT INTO agg_feeds (feed_url, checkFrequency) values (%s,%s)"""
			try:
				self.cursor.execute (addFeedStmt, (urlToAdd, "60"))
				feedID = self.cursor.insert_id()
			except Exception, e:
				logging.warn ("Exception occured adding new feed - feed already present?  Error: %s" % str (e))
				return "Error adding feed, please try again in a few moments."
		else:
			feedID = result[0]
			logging.info ("Found existing feed - just adding this user to the list...")
		addUserStmt = """INSERT INTO agg_user_feeds (user_id, feed_id, userLabel, subscriptionDate) VALUES (%s, %s, %s, %s)"""
		try:
			self.cursor.execute (addUserStmt, (self.userID, feedID, label, mx.DateTime.gmt()))
		except Exception, e:
			logging.warn ("Exception occured subscribing user to feed.  Error: %s" % str (e))
			return "Error adding feed, are you already subscribed to this feed?"
		return None

	def _deleteFeed_ (self, feedID):
		removeSubStmt = """DELETE FROM agg_user_feeds WHERE user_id=%s AND feed_id=%s"""
		try:
			logging.info ("Unsubscribing userID %s from feed %s" % (str (self.userID), str (feedID)))
			self.cursor.execute (removeSubStmt, (self.userID, feedID))
		except Exception, e:
			msg = "Error occured removing subscription: %s" % str (e)
			logging.error (msg)
			return msg
		return None

