""" RSSParser """
import mx.DateTime, mx.DateTime.Parser
import xml.sax, xml.sax.saxutils
import sgmllib, re, codecs

import logging

XML_DECL_REGEX = re.compile ('<\?xml[^\?>].*\?>')
ENCODING_REGEX = re.compile ('encoding="([^"]*)')

class Parser:
	def __init__ (self, rssData):
		self.rssData = rssData
		self.channelDetails = {}
		self.itemList = []
		self.isGoodXML = 0
		self.parsedStatus = 0
		try:
			parser = XMLParser (rssData)
			# If we get this far then this is a good feed!
			self.isGoodXML = 1
			self.channelDetails = parser.channelDetails
			self.itemList = parser.items
			self.parsedStatus = 1
		except Exception, e:
			# Should attempt the same parse using sgmllib at this point.
			#logging.info ("Exception using XML Parser: " + str (e))
			parser = SGMLParser (rssData)
			self.isGoodXML = 0
			if (len (parser.channelDetails) > 0):
				self.parsedStatus = 1
			self.channelDetails = parser.channelDetails
			self.itemList = parser.items
		
	def getItems (self):
		return self.itemList
		
	def getStatus (self):
		return self.parsedStatus
		
	def getChannelDetails (self):
		return self.channelDetails
		
	def getXMLStatus (self):
		if (self.isGoodXML):
			return 'Y'
		else:
			return 'N'
		
	def getBuildDate (self):
		# Return the build date in GMT, SYSDATE if not found
		buildDate = self.channelDetails.get ('lastBuildDate', None)
		if (buildDate is not None):
			return buildDate
		else:
			return mx.DateTime.gmt()
			
class RSSParserBase:
	def __init__ (self):
		self.items = []
		self.channelDetails = {}
		self.currentItem = {}
		self.currentImage = {}
		self.currentData = u""
		self.inItem = 0
		self.inChannelImage = 0
		self.inChannel = 0
		
	def startRSSElement (self, name, attributes):
		# WARNING: name is can be all lower case if the SGML parser is used.
		if ((not self.inItem) and (name == "item")):
			self.inItem = 1
		if ((not self.inChannelImage) and (not self.inItem) and (name == "image")):
			self.inChannelImage = 1
		if ((not self.inItem) and (not self.inChannelImage) and (name == "channel")):
			self.inChannel = 1
			
	def endRSSElement (self, name):
		# WARNING: name is can be all lower case if the SGML parser is used.
		if (self.inItem):
			if (name in ("title", "link", "description", "guid")):
				self.currentItem [name] = self.currentData.strip()
			elif (name.lower() == "pubdate"):
				try:
					pubDate = mx.DateTime.Parser.DateTimeFromString (self.currentData.strip())
					self.currentItem ['pubDate'] = pubDate
				except:
					pass
			elif (name == "item"):
				self.items.append (self.currentItem)
				self.currentItem = {}
				self.inItem = 0
		elif (self.inChannelImage):
			if (name in ("title", "link", "url")):
				self.currentImage [name] = self.currentData.strip()
			elif (name == "image"):
				self.channelDetails [name] = self.currentImage
				self.currentImage = {}
				self.inChannelImage = 0
		elif (self.inChannel):
			if (name in ("title", "link", "language", "description")):
				self.channelDetails [name] = self.currentData.strip()
			elif (name.lower() == "lastbuilddate"):
				try:
					pubDate = mx.DateTime.Parser.DateTimeFromString (self.currentData.strip())
					self.channelDetails ['lastBuildDate'] = pubDate
				except:
					pass
			elif (name == "channel"):
				self.inChannel = 0
		# End of element, so start a new character string
		self.currentData = u""
		
	def RSSCharacters (self, txt):
		self.currentData = self.currentData + txt

		
class XMLParser (xml.sax.handler.ContentHandler, RSSParserBase):
	def __init__ (self, data):
		RSSParserBase.__init__(self)
		self.log = logging.getLogger ("RSSParser.XMLParser")
		self.log.info ("Parsing RSS using XML parser.")
		xml.sax.parseString (data, self)
				
	def startElement (self, name, attributes):
		self.startRSSElement (name, attributes)
		
	def endElement (self, name):
		self.endRSSElement (name)
		
	def characters (self, txt):
		self.RSSCharacters (txt)
	
ENTITY_REF_TABLE = {'lt': '<'
									 ,'gt': '>'
									 ,'quot': '"'
									 ,'apos': "'"
									 ,'amp': '&'
									 }
class SGMLParser (sgmllib.SGMLParser, RSSParserBase):
	def __init__ (self, data):
		sgmllib.SGMLParser.__init__ (self)
		RSSParserBase.__init__ (self)
		self.log = logging.getLogger ("RSSParser.SGMLParser")
		self.log.info ("Parsing RSS using SGML parser.")
		self.decoder = codecs.lookup ("UTF-8")[1]
		# Look for any type specification
		try:
			declMatch = XML_DECL_REGEX.match (data)
			if (declMatch is not None):
				match = ENCODING_REGEX.search (data[declMatch.start(0):declMatch.end(0)])
				if (match is not None):
					encodingName = data[match.start(1):match.end(1)]
					self.decoder = codecs.lookup (encodingName)[1]
					self.log.info ("Using regex found encoding of: " + encodingName)
		except Exception, e:
			self.log.error ("Error (%s) finding encoding using regex, continuing..." % str (e))
			
		self.feed (data)
		self.close()
				
	def unknown_starttag (self, name, attributes):
		self.startRSSElement (name, attributes)
		
	def unknown_endtag (self, name):
		self.endRSSElement (name)
		
	def handle_data (self, txt):
		self.RSSCharacters (self.decoder (txt, 'replace')[0])

	def handle_charref (self, ref):
		# Keep character references - let the browser deal with them.
		self.RSSCharacters (u'&#%s;' % ref)
		
	def handle_entityref (self, ref):
		# Assume that the content we recieve is HTML, it usually is :-(
		# Expand the entity ref, and fall back to keeping it if we don't support it.
		self.RSSCharacters (ENTITY_REF_TABLE.get(ref, u'&%s;' % ref))
	
		