#!/usr/bin/python
""" Copyright 2003 Colin Stewart (http://www.owlfish.com/)
		
		This code is made freely available for commercial and non-commercial use.
		No warranties, expressed or implied, are made as to the fitness of this
		code for any purpose.
		
		If you make any bug fixes or feature enhancements please let me know!
		
		Unit test cases.
		
"""

import unittest, os
import StringIO
import mx.DateTime, mx.DateTime.Parser
import logging, logging.config
import RSSParser

if (os.path.exists ("logging.ini")):
	logging.config.fileConfig ("logging.ini")
else:
	logging.basicConfig()
	
class RSSParserTestCases (unittest.TestCase):
	def setUp (self):
		pass
		
	def testNoData (self):
		data = ""
		parser = RSSParser.Parser (data)
		self.failUnless (not parser.getStatus(), "Parsing should have failed, but passed!")
				
	def testXML091 (self):
		data = """<?xml version="1.0" encoding="ISO-8859-1" ?>
<rss version="0.91">
	<channel>
		<title>0.91 Test</title> 
		<link>http://www.owlfish.com</link> 
		<description>Test case for 0.91 RSS</description> 
		<language>en-gb</language> 
		<copyright>Copyright 2003</copyright> 
		<managingEditor>Me</managingEditor> 
		<webMaster>You</webMaster> 
		<image>
			<title>RSS Test</title> 
			<url>http://www.owlfish.com/</url> 
			<link>http://www.owlfish.com/</link> 
			<width>88</width> 
			<height>31</height> 
			<description>RSS Test</description> 
		</image>
		<lastBuildDate>Sunday, 17 November, 2002, 18:27 GMT</lastBuildDate>
		<item>
			<title>Test Article 1</title> 
			<link>http://test/</link> 
			<description>Test article</description> 
		</item>
		<item>
			<title>Test Article 2</title> 
			<link>http://test/</link> 
			<description>Test article 2</description> 
		</item>
	</channel>
</rss>"""
		parser = RSSParser.Parser (data)
		self.failUnless (parser.getStatus(), "Parsing failed!")
		self.failUnless (parser.getXMLStatus() == "Y", "RSS was not parsed as good XML: " + parser.getXMLStatus())
		self.failUnless (parser.getBuildDate() == mx.DateTime.Parser.DateTimeFromString ('Sunday, 17 November, 2002, 18:27 GMT'), "Last build date not extracted.")
		chanDets = parser.getChannelDetails()
		
		# Image tests
		self.failUnless (chanDets.has_key ('image'), "Image not parsed.")
		image = chanDets['image']
		self.failUnless (image.get ('title', '') == "RSS Test", "Image title not parsed.")
		self.failUnless (image.get ('link', '') == "http://www.owlfish.com/", "Image link not parsed.")
		self.failUnless (image.get ('url', '') == "http://www.owlfish.com/", "Image url not parsed.")
		
		# Channel details tests
		self.failUnless (chanDets.get ('title','') == "0.91 Test", "Channel title not parsed.")
		self.failUnless (chanDets.get ('link','') == "http://www.owlfish.com", "Channel link not parsed.")
		self.failUnless (chanDets.get ('language','') == "en-gb", "Channel language not parsed.")
		self.failUnless (chanDets.get ('description','') == "Test case for 0.91 RSS", "Channel description not parsed.")
		
		# Item tests
		items = parser.getItems()
		self.failUnless (len (items) == 2, "Not all items parsed!")
		self.failUnless (items[0].get('title', '') == "Test Article 1", "Item title not parsed.")
		self.failUnless (items[0].get('link', '') == "http://test/", "Item link not parsed.")
		self.failUnless (items[0].get('description', '') == "Test article", "Item description not parsed.")
		
	def testXML20 (self):
		data = """<?xml version="1.0" encoding="ISO-8859-1" ?>
<rss version="2.0">
	<channel>
		<title>2.0 Test</title> 
		<link>http://www.owlfish.com</link> 
		<description>Test case for 2 RSS</description> 
		<language>en-gb</language> 
		<copyright>Copyright 2003</copyright> 
		<managingEditor>Me</managingEditor> 
		<webMaster>You</webMaster> 
		<image>
			<title>RSS Test</title> 
			<url>http://www.owlfish.com/</url> 
			<link>http://www.owlfish.com/</link> 
			<width>88</width> 
			<height>31</height> 
			<description>RSS Test</description> 
		</image>
		<lastBuildDate>Sunday, 17 November, 2002, 18:27 GMT</lastBuildDate>
		<item>
			<title>Test Article 1</title> 
			<link>http://test/</link> 
			<description>Test article</description> 
			<pubDate>Sunday, 10 November, 2002, 18:27 GMT</pubDate>
			<guid>1</guid>
		</item>
		<item>
			<title>Test Article 2</title> 
			<link>http://test/</link> 
			<description>Test article 2</description> 
			<pubDate>Sunday, 9 November, 2002, 18:27 GMT</pubDate>
			<guid>2</guid>
		</item>
	</channel>
</rss>"""
		parser = RSSParser.Parser (data)
		# Item tests
		items = parser.getItems()
		self.failUnless (len (items) == 2, "Not all items parsed!")
		self.failUnless (items[0].get('guid', '') == "1", "Item guid not parsed.")
		self.failUnless (items[0].get('pubDate', mx.DateTime.Parser.DateTimeFromString ('Sunday, 17 November, 2002, 18:27 GMT')) == mx.DateTime.Parser.DateTimeFromString ('Sunday, 10 November, 2002, 18:27 GMT'), "Item pubDate not parsed.")
		
	def testSGML091 (self):
		data = """<?xml version="1.0" encoding="ISO-8859-1" ?>
<rss version="0.91">
	<channel>
		<title>0.91 Test</title> 
		<link>http://www.owlfish.com</link> 
		<lastBuildDate>Sunday, 17 November, 2002, 18:27 GMT</lastBuildDate>
		<description>Test & case for 0.91 RSS</description> 
		<language>en-gb</language> 
		<copyright>Copyright 2003</copyright> 
		<managingEditor>Me</managingEditor> 
		<webMaster>You</webMaster> 
		<image>
			<title badattribut=a>RSS Test</title> 
			<url>http://www.owlfish.com/</url> 
			<link>http://www.owlfish.com/</link> 
			<width>88</width> 
			<height>31</height> 
			<description>RSS Test</description> 
		</image>
		<item>
			<title>Test Article 1</title> 
			<link>http://test/</link> 
			<description>Test article</description> 
		</item>
		<item>
			<title>Test Article 2</title> 
			<link>http://test/</link> 
			<description>Test article 2</description> 
		</item>
	</channel>
</rss>"""
		parser = RSSParser.Parser (data)
		self.failUnless (parser.getStatus(), "Parsing failed!")
		self.failUnless (parser.getXMLStatus() == "N", "RSS was not parsed as bad XML: " + parser.getXMLStatus())
		self.failUnless (parser.getBuildDate() == mx.DateTime.Parser.DateTimeFromString ('Sunday, 17 November, 2002, 18:27 GMT'), "Last build date not correct, found: " + str (parser.getBuildDate()))
		chanDets = parser.getChannelDetails()
		
		# Image tests
		self.failUnless (chanDets.has_key ('image'), "Image not parsed.")
		image = chanDets['image']
		self.failUnless (image.get ('title', '') == "RSS Test", "Image title not parsed.")
		self.failUnless (image.get ('link', '') == "http://www.owlfish.com/", "Image link not parsed.")
		self.failUnless (image.get ('url', '') == "http://www.owlfish.com/", "Image url not parsed.")
		
		# Channel details tests
		self.failUnless (chanDets.get ('title','') == "0.91 Test", "Channel title not parsed.")
		self.failUnless (chanDets.get ('link','') == "http://www.owlfish.com", "Channel link not parsed.")
		self.failUnless (chanDets.get ('language','') == "en-gb", "Channel language not parsed.")
		self.failUnless (chanDets.get ('description','') == "Test & case for 0.91 RSS", "Channel description not parsed, got back:" + chanDets.get ('description',''))
		
		# Item tests
		items = parser.getItems()
		self.failUnless (len (items) == 2, "Not all items parsed!")
		self.failUnless (items[0].get('title', '') == "Test Article 1", "Item title not parsed.")
		self.failUnless (items[0].get('link', '') == "http://test/", "Item link not parsed.")
		self.failUnless (items[0].get('description', '') == "Test article", "Item description not parsed.")
		
		
if __name__ == '__main__':
	unittest.main()

